#include <b.h>

#define TEST6_MAX 1024

#ifdef WIN32
#define snprintf sprintf_s
#define TEST6_FILE ".\\TEST6"
#else
#define TEST6_FILE "/tmp/test6"
#endif

int
main (int argc, char **argv)
{
  b_t *b;
  b_triple_t *triple;
  b_triple_t *compare;
  char subject[128];
  int subject_type;
  char property[128];
  char object[128];
  int object_type;
  char context[128];
  int context_type;
  char datatype[128];
  int datatype_type;
  char lang[128];
  int lang_type;
  int i, m, c;
  b_error_t err;

  b_iterator_triple_t *iterator;

  fprintf (stderr, "Creating new b struct... ");
  if ((err = b_new (&b, (unsigned char *) TEST6_FILE)) != B_OK)
    {
      fprintf (stderr, "b_new: %s\n", b_strerror (err));
      return 1;
    }

  if (argc > 1 && (c = atoi (argv[1])))
    {
      fprintf (stderr, "done.\nSetting cache to %d... ", c);
      b_set_max_cache (b, c);
    }

  if (argc <= 2 || !(m = atoi (argv[2])))
    m = TEST6_MAX;

  fprintf (stderr, "done.\nStoring %d triples...", m);

  for (i = 0; i < m; i++)
    {
      subject_type = i % 2;
      snprintf (subject, sizeof (subject), "subject %d %s", i % 111,
		!subject_type ? "uri" : "bnode");

      snprintf (property, sizeof (property), "property %d", i);

      object_type = i % 3;
      snprintf (object, sizeof (object), "object %d %s", i % 333,
		!object_type ? "uri" : object_type ==
		1 ? "bnode" : "literal");

      context_type = i % 4;
      snprintf (context, sizeof (context), "context %d", i % 222);

      datatype_type = 1;
      snprintf (datatype, sizeof (datatype), "datatype");

      lang_type = i % 7;
      snprintf (lang, sizeof (lang), "lang %d", i % 222);

      if ((err =
	   b_triple_new (&triple,
			 !subject_type ? (unsigned char *) subject : NULL, 0,
			 !subject_type ? NULL : (unsigned char *) subject, 0,
			 (unsigned char *) property, 0,
			 !object_type ? (unsigned char *) object : NULL, 0,
			 object_type == 1 ? (unsigned char *) object : NULL,
			 0,
			 object_type == 2 ? (unsigned char *) object : NULL,
			 0, context_type ? (unsigned char *) context : NULL,
			 0, datatype_type ? (unsigned char *) datatype : NULL,
			 0, lang_type ? (unsigned char *) lang : NULL,
			 0)) != B_OK)
	{
	  fprintf (stderr, "b_triple_new: %s\n", b_strerror (err));
	  return 1;
	}

      if ((err = b_add_triple (b, triple)) != B_OK)
	{
	  fprintf (stderr, "b_triple_new: %s\n", b_strerror (err));
	  return 1;
	}

      b_triple_destroy (triple);
    }

  fprintf (stderr, "done.\nCreating a incomplete triple...");

  if ((err =
       b_triple_new_incomplete (&compare, NULL, 0, NULL, 0, NULL, 0, NULL, 0,
				NULL, 0, NULL, 0, NULL, 0,
				(unsigned char *) "datatype", 0, NULL,
				0)) != B_OK)
    {
      fprintf (stderr, "b_triple_new_incomplete: %s\n", b_strerror (err));
      return 1;
    }

  fprintf (stderr, "done.\nCreating iterator... ");
  if ((err = b_iterator_triple_new (b, &iterator, compare)) != B_OK)
    {
      fprintf (stderr, "b_iterator_new: %s\n", b_strerror (err));
      return 1;
    }

  fprintf (stderr, "done.\nDestroing the incomplete triple...");
  if ((err = b_triple_destroy (compare)) != B_OK)
    {
      fprintf (stderr, "b_triple_desotry: %s\n", b_strerror (err));
      return 1;
    }

  fprintf (stderr, "done.\n");

  while ((err = b_iterator_triple_step (iterator, &triple)) == B_OK && triple)
    {
      fprintf (stderr, "Next data: ");
      b_triple_print (stderr, triple);
      b_triple_destroy (triple);
    }

  if (err != B_OK)
    {
      fprintf (stderr, "b_iterator_step: %s\n", b_strerror (err));
      return 1;
    }

  fprintf (stderr, "Destroying iterator... ");
  if ((err = b_iterator_triple_destroy (iterator)) != B_OK)
    {
      fprintf (stderr, "b_iterator_destroy: %s\n", b_strerror (err));
      return 1;
    }

  fprintf (stderr, "done.\nDestroing the b struct... ");

  if ((err = b_destroy (b)) != B_OK)
    {
      fprintf (stderr, "b_destroy: %s\n", b_strerror (err));
      return 1;
    }

  fprintf (stderr, "done.\n");
  return 0;
}
